package barrick;

/**
 * An abstract implementation of the ThreadControlable Interface. All that is required
 * to use this class is to implement the doStep() method. The underlying thread, and
 * the ThreadControlable structures are provided. doStop should be overridden to
 * perform the tasks when the ThreadControlable is stopped.
 */
abstract public class ControledThread implements ThreadControlable {

    private boolean m_paused = true;
    private boolean m_stopped = false;
    private boolean m_step = false;
    private Thread m_thread;

    public ControledThread()
    {
        m_thread = new UnderlyingThread();
        m_thread.start();
    }

    public synchronized void run() throws ThreadControlException {
        if(isStopped())
            throw new ThreadControlException("Cannot run a stopped ControledThread");
        if(!isPaused())
            throw new ThreadControlException("Cannot run a running ControledThread");
        setPaused(false);
       notify();
    }

    public synchronized void pause() throws ThreadControlException {
        if(isStopped())
            throw new ThreadControlException("Cannot pause a stopped ControledThread");
        setPaused(true);
    }

    public synchronized void stop() throws ThreadControlException {
      setStopped(true);
      if(isPaused())
          notify();
      setPaused(true);
    }

    /**
     *  Perform actions when this ThreadControlable is stopped.
     */
    protected void doStop()
    {

    }

    abstract public void doStep();

    public synchronized boolean isPaused()
    {
        return m_paused;
    }

    private synchronized void setPaused(boolean _paused)
    {
        m_paused = _paused;
    }

    public synchronized boolean isStopped()
    {
        return m_stopped;
    }

    private synchronized void setStopped(boolean _stopped)
    {
        m_stopped = _stopped;
    }

    private class UnderlyingThread extends Thread
    {
        public void run() {
            while(! isStopped())
            {
                if(isPaused())
                    try {
                        synchronized(ControledThread.this)
                        {
                            ControledThread.this.wait();
                        }
                    } catch (InterruptedException e) {
                        //Ignore this
                    }
                   
                        doStep();

            }
            doStop();
        }
    }

    public synchronized void step() throws ThreadControlException {
        if(isStopped())
            throw new ThreadControlException("Cannot step a stopped ControledThread");
        if(!isPaused())
            throw new ThreadControlException("Cannot step a running ControledThread");
         notify();

    }
}
