package barrick;

/**
 * Define an interface for a thread that is capable of stepping through its
 * execution one unit at a time, and polls a set of control flags between steps
 * allowing the thread to be easily stopped, paused, ran, or (re)started.
 */
public interface ThreadControlable {

    /**
     * Starts a Thread or similar process within the ThreadControlable which
     * executes a step, then checks to see if it should stop/pause, and repeats.
     * @throws ThreadControlException If the object is unable to begin running,
     * i.e. has been stopped or is already running.
     */

    public void run() throws ThreadControlException;

    /**
     * Causes the ThreadControlable object to perform a single step of its
     * computation. The time taken to perform an single step operation is assume
     * to be relatively short or at least well bounded.   <p>
     * Step should be synchronized so that only one thread may be stepping at
     * once.
     *
     * @throws ThreadControlException If the state of the object is such that it
     * cannot step. For example if it is currently running or has been stopped
     */
    public void step() throws ThreadControlException;



    /**
     * Pauses a currently running process after it has completed its current step
     * (if it is currently running).
     * <p>
     * Note if the ThreadControlable is not running and has not stopped, this method
     * will have no effect and return without error.
     *
     * @throws ThreadControlException If the thread cannot be paused, that is
     * because it has been stopped.
     */
    public void pause() throws ThreadControlException;

    /**
     * Tells the ThreadControlable to (after it completes its current step) to
     * stop computation, clean up anything it needs to do. After this method is called
     * a ThreadControlable cannot be caused to execute in any fashion.
     * <p>
     * Note if the ThreadControlable is already stopped, this method has no effect
     * and returns without error
     *
     * @throws ThreadControlException If for some reason the the ThreadControlable
     * cannot stopped. (The author cannot think of why this would happen though).
     *
     */
    public void stop() throws ThreadControlException;

    public boolean isStopped();
    public boolean isPaused();
}
